--[[

States:
-1	Display error (expected: errormsg)
0	Temp main menu (enter state)
1	The editor with an empty level by default
2	Syntax highlighting test
3	Scripting editor
4	Some XML testing
5	Filesystem testing
6	Simple listing of all files in the levels folder

Debug keys:
F12: change state
' (in editor): display all tilesets and colors in the console
\ (in editor): tileset creator/compiler

]]

function love.load()
	require("lang/English")
	require("const")
	require("func")
	require("roomfunc")
	require("vvvvvvxml")
	
	require("filefunc_win")
	
	--libxml = require('xml.init')
	--require "libxml//libxml"
	--libxml = require("xml/init")
	--xmlparsert = require("xml")

	if love.window == nil then
		-- Our version of löve is too old, display error message.
		print("Outdated love2d version!")
		errormsg = L.OUTDATEDLOVE
		state = -1
	else
		cons("love.load() reached")
		
		state = 0; oldstate = "none"
		debug = 0
		input = ""; takinginput = false
		__ = "_"
		
		mousepressed = false -- for some things
		
		tilesets = {}
		
		-- Load a couple of images
		cursorimg = {}
		cursorimg[0] = love.graphics.newImage("cursor/cursor.png")
		cursorimg[1] = love.graphics.newImage("cursor/cursor1.png")
		cursorimg[2] = love.graphics.newImage("cursor/cursor2.png")
		cursorimg[3] = love.graphics.newImage("cursor/cursor3.png")
		cursorimg[4] = love.graphics.newImage("cursor/cursor4.png")
		
		selectedtoolborder = love.graphics.newImage("selectedtool.png")
		unselectedtoolborder = love.graphics.newImage("unselectedtool.png")
		
		savebtn = love.graphics.newImage("save.png")
		loadbtn = love.graphics.newImage("load.png")
		
		bggrid = love.graphics.newImage("bggrid.png")
		
		toolimg = {}
		for t = 1, 17 do
			toolimg[t] = love.graphics.newImage("tools/" .. t .. ".png")
		end
		
		scrollup = love.graphics.newImage("scrollup.png")
		scrolldn = love.graphics.newImage("scrolldn.png")
		
		-- Load the default tilesets
		loadtileset("tiles.png")
		loadtileset("tiles2.png")
		-- loadtileset("tiles3.png") oops oversight
		
		-- eeeeeeeeee
		love.keyboard.setKeyRepeat(true)
	end
end

function love.draw()
	if state == -1 then
		love.graphics.printf(L.FATALERROR .. anythingbutnil(errormsg) .. "\n\n" .. L.FATALEND, 10, 10, love.graphics.getWidth()-20, "left")
	elseif state == 0 then
		love.graphics.print("Placeholder main menu. Enter state: " .. input .. __, 10, 10)
		startinputonce()
	elseif state == 1 then
		-- Ok first of all are we clicking?
		if love.mouse.isDown("l") and mouseon(64, 0, 639, 480) then
			atx = math.floor((love.mouse.getX()-64) / 16)
			aty = math.floor((love.mouse.getY()) / 16)
			
			if tilespicker then
				if tilescreator then
					if not mousepressed then
						if creatorstep == 1 then
							cons("Wall tile " .. creatorsubstep .. ": " .. (aty*40)+(atx+1)-1)
							
							tilesetblocks.creator.colors.creator.blocks[creatorsubstep] = (aty*40)+(atx+1)-1
							cb[creatorsubstep] = (aty*40)+(atx+1)-1
						elseif creatorstep == 2 then
							cons("Background tile " .. creatorsubstep .. ": " .. (aty*40)+(atx+1)-1)
							
							tilesetblocks.creator.colors.creator.background[creatorsubstep] = (aty*40)+(atx+1)-1
							ca[creatorsubstep] = (aty*40)+(atx+1)-1
						elseif creatorstep == 3 then
							cons("Spike tile " .. creatorsubstep .. ": " .. (aty*40)+(atx+1)-1)
							
							tilesetblocks.creator.colors.creator.spikes[creatorsubstep] = (aty*40)+(atx+1)-1
							cs[creatorsubstep] = (aty*40)+(atx+1)-1
						end
						
						if creatorsubstep == 30 then
							creatorstep = creatorstep + 1
							cons("Next: step " .. creatorstep)
							selectedtool = creatorstep
							
							creatorsubstep = 1
						else
							creatorsubstep = creatorsubstep + 1
						end
						
						if creatorstep == 4 then
							spitoutarrays()
							cons("*** ARRAYS COPIED TO CLIPBOARD ***")
							tilescreator = false
						end
						
						mousepressed = true
					end
				else
					cons("Tile selected: " .. (aty*40)+(atx+1)-1)
				
					selectedtile = (aty*40)+(atx+1)-1
				end
			else
				cons("Tile clicked: " .. atx .. " " .. aty .. ", set to " .. selectedtile)
				
				roomdata[roomy][roomx][(aty*40)+(atx+1)] = selectedtile
				
				--[[
				if love.keyboard.isDown("lshift") then
					roomdata[(aty*40)+(atx+1)] = roomdata[(aty*40)+(atx+1)] - 1
				else
					roomdata[(aty*40)+(atx+1)] = roomdata[(aty*40)+(atx+1)] + 1
				end
				]]
			end
		elseif love.mouse.isDown("r") and mouseon(64, 0, 639, 480) then
			atx = math.floor((love.mouse.getX()-64) / 16)
			aty = math.floor((love.mouse.getY()) / 16)
			
			cons("Tile cleared: " .. atx .. " " .. aty)
			
			roomdata[roomy][roomx][(aty*40)+(atx+1)] = 0
		end
		
		if tilespicker then
			displaytilespicker(64, 0, tilesetnames[metadata.tileset])
		else
			love.graphics.draw(bggrid, 64, 0)
			displayroom(64, 0, roomdata[roomy][roomx], metadata)
		end
		
		-- Now display the cursor. If it's on the level
		if mouseon(64, 0, 639, 480) then
			cursorx = math.floor((love.mouse.getX()-64) / 16)
			cursory = math.floor((love.mouse.getY()) / 16)
			
			love.graphics.draw(cursorimg[0], (cursorx*16)+64, (cursory*16))
		else
			cursorx = "--"
			cursory = "--"
		end
		
		-- We also want the tools on the left. But it's a scrollable area.
		love.graphics.setScissor(16, 16, 32, love.graphics.getHeight()-32)
		
		for t = 1, 17 do
			-- love.graphics.rectangle("fill", 16, (16+(48*(t-1)))+lefttoolscroll, 32, 32)
			-- Are we hovering over it? Or maybe even clicking it?
			if ((mouseon(16, 0, 32, 16)) or (mouseon(16, love.graphics.getHeight()-16, 32, 16)) or (not mouseon(16, (16+(48*(t-1)))+lefttoolscroll, 32, 32))) and selectedtool ~= t then
				love.graphics.setColor(255,255,255,128)
			end
			
			if love.mouse.isDown("l") and mouseon(16, (16+(48*(t-1)))+lefttoolscroll, 32, 32) and not mouseon(16, 0, 32, 16) and not mouseon(16, love.graphics.getHeight()-16, 32, 16) then
				selectedtool = t
			end
			
			if selectedtool == t then
				love.graphics.draw(selectedtoolborder,  16, (16+(48*(t-1)))+lefttoolscroll)
			else
				love.graphics.draw(unselectedtoolborder,  16, (16+(48*(t-1)))+lefttoolscroll)
			end
			
			love.graphics.draw(toolimg[t], 16+2, (16+2+(48*(t-1)))+lefttoolscroll)
			love.graphics.setColor(255,255,255,255)
			
			if ((not mouseon(16, 0, 32, 16)) and not (mouseon(16, love.graphics.getHeight()-16, 32, 16)) and (mouseon(16, (16+(48*(t-1)))+lefttoolscroll, 32, 32))) then
				-- Ugh this code but we're hovering over it. So display a tooltip, but don't get snipped away by the scissors.
				love.graphics.setScissor()
				love.graphics.setColor(128,128,128,192)
				love.graphics.rectangle("fill", love.mouse.getX()+15, love.mouse.getY()-10, string.len(toolnames[t])*8, 8)
				love.graphics.setColor(255,255,255,255)
				love.graphics.print(toolnames[t], love.mouse.getX()+16, love.mouse.getY()-8)
				love.graphics.setScissor(16, 16, 32, love.graphics.getHeight()-32)
			end
		end
		
		-- We're done with that now.
		love.graphics.setScissor()
		
		-- But we still want to be able to scroll!
		hoverdraw(scrollup, 16, 0, 32, 16)
		hoverdraw(scrolldn, 16, love.graphics.getHeight()-16, 32, 16)
		
		-- Are we clicking them?
		if love.mouse.isDown("l") and mouseon(16, 0, 32, 16) then
			lefttoolscroll = lefttoolscroll + 4
			lefttoolscrollbounds()
		elseif love.mouse.isDown("l") and mouseon(16, love.graphics.getHeight()-16, 32, 16) then
			lefttoolscroll = lefttoolscroll - 4
			lefttoolscrollbounds()
		end
		
		-- Now stuff on the right.
		hoverdraw(loadbtn, love.graphics.getWidth()-64, 0, 32, 32)
		hoverdraw(savebtn, love.graphics.getWidth()-32, 0, 32, 32)
		
		-- And coordinates.
		love.graphics.print("(" .. (roomx+1) .. "," .. (roomy+1) .. ")", love.graphics.getWidth()-56, love.graphics.getHeight()-16)
		
		-- But if we're in the tiles picker instead display the tile we're hovering on!
		if tilespicker then
			if (cursorx ~= "--") and (cursory ~= "--") then
				love.graphics.print(L.TILE .. " " .. (cursory*40)+(cursorx+1)-1, love.graphics.getWidth()-72, love.graphics.getHeight()-8)
			else
				love.graphics.print(L.TILE .. " ----", love.graphics.getWidth()-72, love.graphics.getHeight()-8)
			end
		else
			love.graphics.print("(" .. cursorx .. "," .. cursory .. ")", love.graphics.getWidth()-56, love.graphics.getHeight()-8)
		end
		
		-- Also display a smaller tiles picker for semi-undirect mode
		love.graphics.rectangle("fill", love.graphics.getWidth()-(7*16)-1, love.graphics.getHeight()-(6*16)-16-24-12-1, (6*16)+2, (5*16)+2)
		love.graphics.setColor(0,0,0,255)
		love.graphics.rectangle("fill", love.graphics.getWidth()-(7*16), love.graphics.getHeight()-(6*16)-16-24-12, (6*16), (5*16))
		love.graphics.setColor(255,255,255,255)
		displaysmalltilespicker(love.graphics.getWidth()-(7*16), love.graphics.getHeight()-(6*16)-16-24-12, selectedtileset, selectedcolor)
		
		-- And text below it.
		--[[
		hoverrectangle(128,128,128,128, love.graphics.getWidth()-(7*16)-1, love.graphics.getHeight()-16-32-2, (6*16), 8)
		love.graphics.print("<tiles2.png>", love.graphics.getWidth()-(7*16), love.graphics.getHeight()-16-32)
		]]
		
		hoverrectangle(128,128,128,128, love.graphics.getWidth()-(7*16)-1, love.graphics.getHeight()-16-32-2-12, (6*16), 8+4)
		love.graphics.print(" " .. selectedtileset, love.graphics.getWidth()-(7*16), love.graphics.getHeight()-16-32-12+2)
		love.graphics.print("v", love.graphics.getWidth()-(1.5*16), love.graphics.getHeight()-16-32-12+2)
		
		hoverrectangle(128,128,128,128, love.graphics.getWidth()-(7*16)-1, love.graphics.getHeight()-16-24-2-8, (6*16), 8+4)
		love.graphics.print(" " .. selectedcolor, love.graphics.getWidth()-(7*16), love.graphics.getHeight()-16-24-8+2)
		love.graphics.print("v", love.graphics.getWidth()-(1.5*16), love.graphics.getHeight()-16-24-8+2)
		
		hoverrectangle(128,128,128,128, love.graphics.getWidth()-(7*16)-1, love.graphics.getHeight()-16-16-2-4, (6*16), 8+4)
		love.graphics.print(tilespicker and L.HIDEALL or L.SHOWALL, love.graphics.getWidth()-(7*16), love.graphics.getHeight()-16-16-4+2)
		
		-- Dropdown for tileset?
		if dropdown == 1 then
			love.graphics.print([[-1-
 Space St.
-2-
 Outside
 Lab
 Warp Zone
 Ship]], love.graphics.getWidth()-(7*16)-1, love.graphics.getHeight()-16-32-2-12-56)
		elseif dropdown == 2 then
		
		end
	elseif state == 2 then
		love.graphics.print("Syntax highlighting" .. input .. __, 10, 10)
		
		syntaxhl(input, 10, 30, false, true)
		startinputonce()
	elseif state == 3 then
		love.graphics.setColor(255,255,255,255)
		love.graphics.print(L.SCRIPTEDITOR .. " - script1", 8, 8)
		love.graphics.print(L.FILE .. "   " .. L.EDIT .. "   " .. L.INSERT .. "   " .. L.HELP, 8, 24)
		-- Script display starts at 8, 48
		--for k,v in pairs(scriptlines) do
		
		-- This can roll over, prevent that!
		textlinestogo = 0
		
		-- Make sure to display all lines but if we put the cursor further, then do display line numbers
		for k = 1, math.max(table.maxn(scriptlines), editingline) do
			v = anythingbutnil(scriptlines[k])
			
			if editingline == k then
				love.graphics.setColor(255,255,255,255)
			else
				love.graphics.setColor(128,128,128,255)
			end
			love.graphics.print(fixdig(k, 3), 8, 48+(8*k))
			
			
			textq = syntaxhl(v, 48, 48+(8*k), textlinestogo > 0, editingline == k)
			
			if textq ~= nil then
				textlinestogo = textq
			elseif textlinestogo > 0 then
				textlinestogo = textlinestogo - 1
			end
		end
	elseif state == 4 then
		--love.graphics.print(xmldoc.documentElement.childNodes[1].getAttribute("name"), 10, 10)
		love.graphics.print(metadata.Creator, 10, 10)
		love.graphics.print("That should say Unknown.", 10, 20)
	elseif state == 5 then
		love.graphics.print("Userprofile: " .. userprofile, 8, 8)
		for k,v in pairs(files) do
			love.graphics.print(v, 8, 16+8*k)
			
			lastk = k
		end
		
		--love.graphics.print(userprofile .. "\\Documentus" .. (directory_exists("C:\\Users\\" .. username .. "\\Documentus") and " EXISTS" or " DOES NOT EXIST"), 8, 16+8*lastk+16)
		--love.graphics.print(userprofile .. "\\My Documentus" .. (directory_exists("C:\\Users\\" .. username .. "\\My Documentus") and " EXISTS" or " DOES NOT EXIST"), 8, 16+8*lastk+24)

		--love.graphics.print(userprofile .. "\\Documents" .. anythingbutnil(userdatathing(directory_exists(userprofile .. "\\Documents"))), 8, 16+8*lastk+16)
		--love.graphics.print(userprofile .. "\\My Documents" .. anythingbutnil(userdatathing(directory_exists(userprofile .. "\\My Documents"))), 8, 16+8*lastk+24)
		
		love.graphics.print("Levels folder error: " .. lerror .. " (0 means no error)", 8, 16+8*lastk+16)
		love.graphics.print("Levels folder: " .. anythingbutnil(levelsfolder), 8, 16+8*lastk+24)

		else
		fatalerror(langkeys(L.UNKNOWNSTATE, {state, oldstate}))
	end
	
	-- Show debug code if wanted
	if allowdebug then
		
	end
end

function love.update()
	if state ~= -1 then
		love.window.setTitle("*Untitled - VVVed  [FPS: " .. love.timer.getFPS() .. "] - State: " .. state .. " - " .. love.graphics.getWidth() .. "x" .. love.graphics.getHeight() .. " Mouse: " .. love.mouse.getX() .. " " .. love.mouse.getY())
	end
end

function love.textinput(char)
	if takinginput then
		input = input .. char
		
		if state == 3 then
			scriptlines[editingline] = input
		end
	end
end

function love.keypressed(key)
	if takinginput then
		if key == "backspace" then
			input = backspace(input)
			
			if state == 3 then
				scriptlines[editingline] = input
			end
		end
	end
	
	if (state == 0) and (key == "return") then
		stopinput()
		tostate(input)
	elseif (state == 1) and (key == "z") then -- temporary key, might change
		zoomscale = 1/2
	elseif (state == 1) and (key == "s") then -- again, but cycle tileset
		metadata.tileset = cycle(metadata.tileset, 2)
	elseif (state == 1) and (key == "t") then -- ^
		tilespicker = not tilespicker
	elseif (state == 1) and (key == "right") then
		-->
		if roomx+1 >= metadata.mapwidth then
			roomx = 0
		else
			roomx = roomx + 1
		end
	elseif (state == 1) and (key == "left") then
		--<
		if roomx+1 <= 1 then
			roomx = metadata.mapwidth-1
		else
			roomx = roomx - 1
		end
	elseif (state == 1) and (key == "down") then
		--v
		if roomy+1 >= metadata.mapheight then
			roomy = 0
		else
			roomy = roomy + 1
		end
	elseif (state == 1) and (key == "up") then
		--^
		if roomy+1 <= 1 then
			roomy = metadata.mapheight-1
		else
			roomy = roomy - 1
		end
	elseif allowdebug and (state == 1) and (key == "\\") then
		cons("*** TILESET COLOR CREATOR STARTED FOR TILESET " .. metadata.tileset .. " ***")
		cons("First select the wall tiles")
		
		tilescreator = true
		cb = {}
		ca = {}
		cs = {}
		creatorstep = 1
		creatorsubstep = 1
		
		selectedtileset = "creator"
		selectedcolor = "creator"
		
		tilesetblocks.creator.tileimg = metadata.tileset
		
		tilespicker = true
		selectedtool = 1
		
		mousepressed = false
	elseif allowdebug and (state == 1) and (key == "'") then
		-- Just display all tilesets and colors in the console.
		for k,v in pairs(tilesetblocks) do
			cons("==== " .. k .. " ====")
			for k2,v2 in pairs(v.colors) do
				cons("-> " .. k2)
			end
		end
	elseif (state == 3) and (key == "up") and (editingline > 1) then
		editingline = editingline - 1
		input = anythingbutnil(scriptlines[editingline])
	elseif (state == 3) and ((key == "down") or (key == "return")) then
		editingline = editingline + 1
		input = anythingbutnil(scriptlines[editingline])
	elseif allowdebug and (key == "f12") then
		tostate(0)
	end
end

function love.mousepressed(x, y, button)
	if state == 1 then
		if x < 64 then
			if button == "wu" then
				lefttoolscroll = lefttoolscroll + 16
				lefttoolscrollbounds()
			elseif button == "wd" then
				lefttoolscroll = lefttoolscroll - 16
				lefttoolscrollbounds()
			end
		elseif mouseon(love.graphics.getWidth()-(7*16)-1, love.graphics.getHeight()-16-32-2-12, (6*16), 8+4) then -- tileset picker
			if dropdown == 1 then
				dropdown = 0
			else
				dropdown = 1
			end
		elseif mouseon(love.graphics.getWidth()-(7*16)-1, love.graphics.getHeight()-16-16-2-4, (6*16), 8+4) then -- show all tiles
			tilespicker = not tilespicker
		end
	end
end

function love.mousereleased()
	mousepressed = false
end